#include <windows.h>

#include "RemoteThread.h"
#include "win9x.h"


#ifdef DEBUG
#include <stdio.h>


/*
	Function to show error message
*/

int DisplayError(DWORD dwError)
{
	LPVOID lpBuffer;
	int retValue;
	
	//did the caller include a error message? 
	if (!dwError) dwError = GetLastError();

	retValue = FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		dwError,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&lpBuffer,
		0,
		NULL
		);

	if (retValue != 0) printf("Error message:%s",(char *)lpBuffer);

	if (lpBuffer) LocalFree(lpBuffer);

	return retValue;
}
#endif

/*
	Shell code to execute a thread and to restore eip
*/


BYTE xRemoteThreadCode[] = 
	"\x00\x00\x00\x00"		//dwId
	"\x00\x00\x00\x00"		//hThread
	"\x60"					//pushad
	"\x68\x00\x00\x00\x00"	//push dwThreadId
	"\x68\x00\x00\x00\x00"	//push 0 
	"\x68\xab\xab\xab\xab"	//push param
	"\x68\xab\xab\xab\xab"	//push startaddress
	"\x68\x00\x00\x00\x00"	//push 0 
	"\x68\x00\x00\x00\x00"	//push 0 
	"\xe8\xab\xab\xab\xab"	//call Kernel32:CreateThread
	"\xa3\x00\x00\x00\x00"	//mov hThread eax
	"\x61"					//popad
	"\xe9\x00\x00\x00\x00"; //jump old eip

/*
	Function creates a thread that runs in the address space of another process.  

	parameters:

hThread	
	Handle to a thread in the remote process used to execute the new thread
	Win NT: The handle must have the THREAD_SET_CONTEXT and THREAD_GET_CONTEXT and THREAD_SUSPEND_RESUME
	access right to the thread

hProcess
	Handle to the remote process 
	Win NT: The handle must have PROCESS_VM_WRITE and PROCESS_VM_OPERATION and PROCESS_VM_READ  
	access to the process. 

dwThreadId
	Id of the thread 

lpStartAddress
	The starting address of the new thread. 
	This is typically the address of a function declared with the WINAPI calling convention that accepts a single 32-bit pointer as an argument and returns a 32-bit exit code. 
	Its prototype is: DWORD WINAPI ThreadFunc( LPVOID );
	Note: 
		Winnt: lpStartAddress must be in the address space of the remote process!!
		Win9x: lpStartAddress must be in shared memory!!

lpParameter
	Specifies a single 32-bit parameter value passed to the thread. 
	Note: 
		Winnt: lpParameter must be in the address space of the remote process!!
		Win9x: lpParameter must be in shared memory!!


lpThreadId
	Points to a 32-bit variable that receives the thread identifier. 

Returns
	If the function succeeds, the return value is a handle to the new thread valid in the calling process.
	If the function fails, the return value is NULL.

Remarks: 
	You should close the returned handle with a call to CloseHandle if you dont need the handle anymore 
*/



HANDLE xCreateRemoteThread(
	HANDLE					hThread,
	HANDLE					hProcess,
	DWORD					dwThreadId,
	LPTHREAD_START_ROUTINE	lpStartAddress,
	LPVOID					lpParameter,
	LPDWORD					lpThreadId
) {

	PBYTE	lpCode=NULL,
			lpRemoteMem=NULL;

	HANDLE	hNewThread,
			hNewThreadRet=NULL;

	DWORD	dwSaveEip;

	*lpThreadId=0;



	CONTEXT ThreadContext;
	
	memset(&ThreadContext,0,sizeof(CONTEXT));

	/* 
		allocate memory for local code
	*/

	lpCode=(LPBYTE)VirtualAlloc(NULL,sizeof(xRemoteThreadCode)-1,MEM_COMMIT,PAGE_EXECUTE_READWRITE);

	if (!lpCode) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Error Allocating memory \"VirtualAlloc\"\n");
#endif
		return 0;
	}
	
	/*
		Copy RemoteThreadCode to lpCode
	*/

	memcpy(lpCode,xRemoteThreadCode,sizeof(xRemoteThreadCode)-1);
	
	/*
		Create memory in the remote proc
	*/

	lpRemoteMem=(PBYTE)xVirtualAllocEx(hProcess,sizeof(xRemoteThreadCode)-1);

	if (!lpRemoteMem) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Could not allocate remote memory \"xVirtualAllocEx\"\n");
#endif
		goto end;
	}

	/*
		Copy parameters to lpCode
	*/

	*(LPDWORD)(lpCode+10)=(DWORD)lpRemoteMem;//addr dwThreadId 
	*(LPDWORD)(lpCode+15)=(DWORD)0;
	*(LPDWORD)(lpCode+20)=(DWORD)lpParameter;
	*(LPDWORD)(lpCode+25)=(DWORD)lpStartAddress;
	*(LPDWORD)(lpCode+40)=((DWORD)GetProcAddress(GetModuleHandle("kernel32"),"CreateThread")-(int)lpRemoteMem-44);
	*(LPDWORD)(lpCode+45)=(DWORD)lpRemoteMem+4;//addr hThread
	

	/*
		Halt the thread
	*/

	if (SuspendThread(hThread) == 0xFFFFFFFF) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Could not Suspend the thread \"SuspendThread\"\n");
#endif
		goto end;
	}
	
	/*
		Get thread context
	*/

	ThreadContext.ContextFlags = CONTEXT_FULL;

	if (!GetThreadContext(hThread,&ThreadContext)) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Could not get thread context \"GetThreadContext\"\n");
#endif
		ResumeThread(hThread);
		goto end;
	}
	
	
#ifdef DEBUG
	printf("Old eip:%0.8x\n",ThreadContext.Eip);
#endif


	/*
		copy address to jump to after the new thread is created
	*/

	*(LPDWORD)((int)lpCode+51) = ThreadContext.Eip - ((int)lpRemoteMem + 55);

	/*
		Copy code to create a new thread to the remote process
	*/

	if (!xWriteProcessMemory(hProcess,lpRemoteMem,lpCode,sizeof(xRemoteThreadCode)-1)) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Could not copy memory to remote process \"xWriteProcessMemory\"\n");
#endif
		goto end;
	}

	/*
		Set thread context Eip to the new code
	*/

	ThreadContext.Eip = (DWORD)lpRemoteMem+8;

	/*
		Save Eip
	*/

	dwSaveEip=ThreadContext.Eip;

#ifdef DEBUG
	printf("Set eip:%0.8x\n",dwSaveEip);
#endif

	ThreadContext.ContextFlags = CONTEXT_CONTROL;

	if (!SetThreadContext( hThread, &ThreadContext )) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Error Set Thread context \"SetThreadContext\"\n");
#endif
		ResumeThread(hThread);
		goto end;
	}

	/*
		Post a message to the thread if it has a message queue
		its not really needed but this way you dont have to waith until your code gets executed
	*/

	if (PostThreadMessage(dwThreadId,WM_USER+100000,0,0) == 0) {
#ifdef DEBUG
		printf("Could not post a message to the threads message queue\n");
		DisplayError(NULL);
#endif
	}

	/*
		resume the thread
	*/
	
	ResumeThread(hThread);

	/*
		we could remove the code below if we dont need the Thread id or handle
		and we cant free the memory if we skip this
		Note if some of the code below fails this will not say the thread isnt executed
	*/

	while (1) {

		/*
			Halt the thread
		*/

		if (SuspendThread(hThread) == 0xFFFFFFFF) {
#ifdef DEBUG
			DisplayError(NULL);
			printf("Could not Suspend the thread \"SuspendThread\"\n");
#endif
			goto end;
		}

		memset(&ThreadContext,0,sizeof(CONTEXT));

		ThreadContext.ContextFlags = CONTEXT_CONTROL;

		/*
			Get the thread context
		*/

		if (GetThreadContext(hThread,&ThreadContext) == FALSE) {
#ifdef DEBUG
			DisplayError(NULL);
			printf("Could not get thread context \"GetThreadContext\"\n");
#endif
			ResumeThread(hThread);

			goto end;
		}

		/*
			Resume the thread
		*/

		ResumeThread(hThread);
		
		/*
			if Eip is changed the jump is executed
			so the new thread is started to
		*/

		if (ThreadContext.Eip != dwSaveEip) {
			break;
		}
		
		/*
			Give the thread some time to execute the code
		*/

		Sleep(10);
	}

#ifdef DEBUG
	printf("Thread new eip is:%0.8x code executed\n",ThreadContext.Eip);
#endif

	/* 
		read first 8 bytes of lpRemoteMem
		first 4 bytes thread id
		next  4 bytes handle to thread
		Note: Handle is in the context of the remote process
	*/
	
	if (!xReadProcessMemory(hProcess,lpRemoteMem,lpCode,8)) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Could not read from remote process \"xReadProcessMemory\"\n");
#endif
		goto end;

	}
	
	hNewThread=*(HANDLE *)((int)lpCode+4);

	if (hNewThread == NULL) {

#ifdef DEBUG
		printf("Error handle to thread is invalid\n");
#endif
		goto end;
	}

	*lpThreadId=(DWORD)*(DWORD *)lpCode;//Thread id

	if (*lpThreadId == 0) {
#ifdef DEBUG
		printf("Error Thread Id is invalid\n");
#endif
		goto end;
	}

	/*
		Get a handle to the thread valid in this process
	*/

	if (DuplicateHandle(
		hProcess,
		hNewThread,
		GetCurrentProcess(),
		&hNewThreadRet,	
		0, 
		FALSE, 
		DUPLICATE_SAME_ACCESS | DUPLICATE_CLOSE_SOURCE
		) == 0) {
#ifdef DEBUG
		DisplayError(NULL);
		printf("Could not duplicate Thread handle \"DuplicateHandle\"\n");
#endif
		hNewThreadRet=NULL;
		goto end;
	}

#ifdef DEBUG
	printf("New Thread id:%i\nLocal Handle to the new Thread:%0.8x\n",*lpThreadId,hNewThreadRet);
#endif


end:;

	//Free the memory and return Thread id
	if (lpCode) VirtualFree(lpCode,0,MEM_RELEASE);

	if (lpRemoteMem) xVirtualFreeEx(hProcess,lpRemoteMem);
	
	return hNewThreadRet;
}