#include <windows.h>
#include "win9x.h"

/*
	if defined DEBUG print debug message's
*/
//#define DEBUG 

#ifdef DEBUG
#include <stdio.h>
#endif


/*
	on win9x there is no VirtualAllocEx, VirtualFreeEx
	but we can allocate memory in the shared memory space < 0x80000000
	This is not possible with normal documented api's but we can use VxDCall 

	WriteProcessMemory and ReadProcessMemory do exists on win9x but we allocate the memory 
	in the shared memory space so we dont need it we just call memcpy
*/


void *xVirtualAllocEx(HANDLE hProc,DWORD dwSize)
{
	if (GetVersion() < 0x80000000) {
		return VirtualAllocEx(hProc,NULL,dwSize,MEM_COMMIT,PAGE_EXECUTE_READWRITE);
	}
	return Win9xAlloc(dwSize);
}

int xVirtualFreeEx(HANDLE hProcess,LPVOID lpAddress)
{
	if (GetVersion() < 0x80000000) return VirtualFreeEx(hProcess,lpAddress,0,MEM_RELEASE);
	return (int)Win9xFree(lpAddress);
}

int xWriteProcessMemory(HANDLE hProcess,LPVOID lpBaseAddress,LPVOID lpBuffer,DWORD nSize)
{
	if (GetVersion() < 0x80000000) {
		DWORD dwBytesWritten;
		if (WriteProcessMemory(hProcess,lpBaseAddress,lpBuffer,nSize,&dwBytesWritten) == 0) return 0;
		if (nSize != dwBytesWritten) return 0;
		return dwBytesWritten;
	}
	else {
		if (memcpy(lpBaseAddress,lpBuffer,nSize)) return 1;
	}
	return 0;
}

int xReadProcessMemory(HANDLE hProcess,LPVOID lpBaseAddress,LPVOID lpBuffer,DWORD nSize)
{
	if (GetVersion() < 0x80000000) {
		DWORD dwBytesRead;
		if (ReadProcessMemory(hProcess,lpBaseAddress,lpBuffer,nSize,&dwBytesRead) == 0) return 0;
		if (nSize != dwBytesRead) return 0;
		return dwBytesRead;
	}
	
	memcpy(lpBuffer,lpBaseAddress,nSize);
	
	return 1;
}

/*
	Function returns the address of the specified exported dynamic-link library (DLL) function by ordinal.  
*/

FARPROC xGetProcAddress(HMODULE hModule,int Ordinal)
{

    IMAGE_NT_HEADERS *inh;
    IMAGE_EXPORT_DIRECTORY *exp;
    
    LPBYTE		pbuf;
	DWORD		*AddrFunc;
	char		**lpNameOfFunc;

    pbuf = (BYTE *)hModule;

	// check magic id 'MZ'
	if (*(WORD *)pbuf != IMAGE_DOS_SIGNATURE) {
		return NULL;
	}

	//move to the PE exe header
	pbuf += ((PIMAGE_DOS_HEADER)pbuf)->e_lfanew;

	// check PE signature 'PE00'
	if (*(DWORD *)pbuf != IMAGE_NT_SIGNATURE) {
		return NULL;
	}

	inh=(IMAGE_NT_HEADERS *)pbuf;

	exp=(IMAGE_EXPORT_DIRECTORY *)(((DWORD)hModule) + (DWORD)inh->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress);

	AddrFunc = (DWORD *) ((DWORD)hModule + (DWORD) exp->AddressOfFunctions);

	lpNameOfFunc=(char **)((DWORD)hModule+(DWORD)exp->AddressOfNames);
	

	if (Ordinal < 1 || Ordinal > (int)exp->NumberOfFunctions) return NULL;
		
	return (FARPROC)((DWORD)hModule + (DWORD)AddrFunc[Ordinal -1]);
}


pVxDPageReserve PageReserve;
pVxDPageCommit  PageCommit;
pVxDPageFree	PageFree;


/*
	Function to allocate shared memory

	iSize = size of the memory

	returns address of memory or NULL if it fails
*/

void *Win9xAlloc(int iSize)
{
	if (!PageReserve || !PageCommit) {
		PageCommit=(pVxDPageCommit)xGetProcAddress(GetModuleHandle("Kernel32"),1);
		PageReserve=(pVxDPageReserve)PageCommit;
		if (!PageReserve) {
#ifdef DEBUG
			printf("error loading VxDCallgate\n");
			return NULL;
#endif
		}
	}
	
	void *pMem=NULL;
	pMem=(void *)PageReserve(0x00010000,PR_SHARED,iSize,PC_WRITEABLE | PC_USER);

	if ((DWORD)pMem == 0x0FFFFFFFF || (DWORD)pMem <= 0x80000000) {
#ifdef DEBUG
		printf("Error Allocate Shared memory PageReserve\n");
#endif
		return NULL;
	}

#ifdef DEBUG
	printf("Memory allocated add address: %0.8X\n",pMem);
#endif

	void *pRet=pMem;

	__asm {
		shr pMem, 0x0C;
	}
	
	if (PageCommit(
			0x00010001,
			(DWORD)pMem,
			iSize,
			PD_ZEROINIT,
			0,
			PC_WRITEABLE | PC_USER | PC_PRESENT | PC_FIXED
		) == 0) return NULL;
	return pRet; 
}

/*
	Function to free shared memory

	lpBase = Start address of memory

	returns NULL if not successfull
*/

DWORD Win9xFree(void *lpBase)
{
	if (!lpBase) return NULL;

	if (!PageFree) {
		PageFree=(pVxDPageFree)xGetProcAddress(GetModuleHandle("kernel32"),1);
		if (!PageFree) {
#ifdef DEBUG
			printf("error loading VxDCallgate\n");
			return NULL;
#endif
		}
	}

	return PageFree(0x0001000a,lpBase,0);

	return true;
}