#include <stdio.h>
#include <windows.h>
#include "psapi.h"
#include "detours.h"
#define PROCESSES_MAXCOUNT 10*1024 //our limit is 10 kilo running processes
#define DelayForSystemReadingStartupInfo_SECONDS 5
#define DelayForUserReadingOutput_SECONDS 2



int SetDebugPrivileges(void)
//thanks to:
//	Alexander Volynkin
//copied from:
//	www.volynkin.com
//check:
//	archive\document\devel\SetDebugPrivileges.htm
//
{
	TOKEN_PRIVILEGES Debug_Privileges;

	//STEP 1
	if (!LookupPrivilegeValue(NULL, // Privieleges for the local system
				SE_DEBUG_NAME, // define the name of the privilege 
                                &Debug_Privileges.Privileges[0].Luid)) // will get the LUID value into this variable
   	{	//if function failed, cannot proceed to the next step
		return GetLastError(); //terminate the outer function
   	}

	//STEP 2
	DWORD err = 0; // define error holder, used to store the error code in case of failure
	HANDLE hToken = 0; //  instantiate a token handle 
	if (!OpenProcessToken (GetCurrentProcess (), // current process ID handle
                           TOKEN_ADJUST_PRIVILEGES, //set the desired access
                           &hToken)) // handle to the token will be held here 
   	{	// if function failed, cannot proceed to the next step
		err = GetLastError();  
		if (hToken) // if handle is still valid
        	CloseHandle (hToken); // destroy it
		return err; //terminate the outer function
   	}

	//STEP3
	Debug_Privileges.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; // set to enable privilege
	Debug_Privileges.PrivilegeCount = 1; // working with only one privilege

	if (!AdjustTokenPrivileges (hToken, // access token handle
								FALSE, // do not disable privileges
								&Debug_Privileges, // pointer to the token structure
								0,  // no need for a buffer 
								NULL, // previous state not set
								NULL)) //  no need for a buffer
   	{
        	err = GetLastError();
		if (hToken) // if handle is still valid
        		CloseHandle (hToken); // destroy it
		return err; //terminate the outer function
   	}
	return err; 
}
char * GetMainModuleFileName(char * sMainModuleFileName, DWORD nSize)
{	
	DWORD dwRet = GetModuleFileNameEx(
  		GetCurrentProcess(), //GetCurrentProcess function retrieves a pseudo handle for the current process. no need to close this handle
  		NULL, //hModule :[in] Handle to the module. If this parameter is NULL, GetModuleFileNameEx returns the path of the executable file of the process specified in hProcess. 
  		sMainModuleFileName, 
  		nSize
		);	
	//If the function fails, the return value is zero
	if(dwRet == 0)
	{
		sMainModuleFileName[0] = 0;
		return sMainModuleFileName;
	}
	
	return sMainModuleFileName;
	
}
BOOL FileExists(char * sFileName)
{
	WCHAR wsFileName[MAX_PATH] = L"";
	int nRet = MultiByteToWideChar(
		CP_ACP, // code page
		MB_PRECOMPOSED, // character-type options , default value is used here.
		sFileName, // string to map
		-1, // number of bytes in string
		wsFileName,  // wide-character buffer
		MAX_PATH // size of buffer
		);
	//If the function fails, the return value is zero. 
	if(nRet == 0)
		return FALSE;//sFileName is longer than MAX_PATH. so it does not exist
		
	HANDLE hTemp = CreateFileW(wsFileName,   // open 
			GENERIC_READ,                // open for reading
			FILE_SHARE_READ|FILE_SHARE_WRITE,              // allow multiple readers 
			NULL,                         // no security 
			OPEN_EXISTING,                  // open or create 
			FILE_ATTRIBUTE_NORMAL,        // normal file 
			NULL);                       // no attr. template 
	if(hTemp == INVALID_HANDLE_VALUE)
	//createfile: If the function fails, the return value is INVALID_HANDLE_VALUE
		return FALSE;
	else
	{
		CloseHandle(hTemp);
		return TRUE;
	}
}



void DelayForSystemReadingStartupInfo(void)
//just delay for two seconds in a non-boring way
{
	
	DWORD i = 0;
	for(i=0;i<DelayForSystemReadingStartupInfo_SECONDS*10;i++)
	{
		Sleep(1000*0.1);
		printf(".");
	}
	printf("\n");
}
int main(void) 
{
	//print logo info.
	printf("\n");
	printf("WinBlox Setup Console for Windows 2000, Windows Xp and Windows 2003\n");
	printf("Copyright (C) Umbrella Security Research 2005. All rights reserved.\n");
	printf("Project Homepage: http://umbrella.name/upid/winblox\n");
	printf("\n");
	
	//check if it's WINDOWS NT series
	DWORD dwVersion = GetVersion();
	if(dwVersion >= 0x80000000)
	{
		printf("FATAL ERROR: You are running Windows Me/98/95.\n");
		return 1;
	}
	
	//get full path of this program
	char filename[MAX_PATH] = "";
	GetMainModuleFileName(filename,MAX_PATH);
	
	//get folder path - no ending slash
	unsigned int i = 0;
	for(i=strlen(filename)-1;i>0;i--)
	if(filename[i]=='\\')
	{
		filename[i]=0;
		break;
	}
	
	//check dll file
	if(  strlen(filename)+strlen("\\INSIDER.dll")<MAX_PATH  )
		strcat(filename,"\\INSIDER.dll");
	else
	{
		printf("FATAL ERROR: Path is too long.\n");
		return 1;
	}
	if(!FileExists(filename))
	{
		printf("FATAL ERROR: \"INSIDER.dll\" is missing.\n");
		return 1;
	}
	
	//get a little more privilege. i found it's required for dll injection into system service
	int SetDbgRet = SetDebugPrivileges();
	printf("SetDebugPrivileges: %08x\n",SetDbgRet);
	
	//get PID of each running process
	DWORD dwProcIdArray[PROCESSES_MAXCOUNT];
	DWORD dwBytesReturned;
	BOOL bEnumProcRet = EnumProcesses(
  				dwProcIdArray, //[out] Pointer to an array that receives the list of process identifiers.
  				PROCESSES_MAXCOUNT*sizeof(DWORD), //[in] Size of the pProcessIds array, in bytes. 
  				&dwBytesReturned //[out] Number of bytes returned in the pProcessIds array. 
				);
	if(bEnumProcRet==FALSE)
	{
		printf("FATAL ERROR: GetLastError for EnumProcesses returned 0x%08x\n",GetLastError());
		return 1;
	}
	if(dwBytesReturned==PROCESSES_MAXCOUNT*sizeof(DWORD))
	//if pBytesReturned equals cb, consider retrying the call with a larger array. ==> array is too small to store all process id
	{
		printf("FATAL ERROR: PROCESSES_MAXCOUNT = 0x%08x is too small.\n",PROCESSES_MAXCOUNT);
		return 1;
	}
	//Usually we block access to STARTUP folder and STARTUP registry
	//Usually CONSOLE.exe is run at the STARTUP stage of system when system does need to access STARTUP info.
	//So we delay for a while before DLL injection, and system will be able to access STARTUP info during this delay
	DelayForSystemReadingStartupInfo();
	//DLL INJECTION
	BOOL bDllInjectionSuccess = FALSE;
	BOOL bGotFailure = FALSE;
	HANDLE hProc = NULL;
	for(i=0;   i<(dwBytesReturned/sizeof(DWORD))-1;   i++)
	if(dwProcIdArray[i]!=0)//SKIP System Idle Process WHOSE PID IS EQUAL TO ZERO
	{
		bDllInjectionSuccess = FALSE;
		hProc = NULL;
		
		printf("Trying DLL injection for PID = 0x%08x = %d ... ",
				dwProcIdArray[i],
				dwProcIdArray[i]
				);
		hProc = OpenProcess(PROCESS_ALL_ACCESS,FALSE,dwProcIdArray[i]);
		if(hProc != NULL)
		//openprocess: If the function fails, the return value is NULL. 
		{
			if(DetourContinueProcessWithDll(hProc,filename)==TRUE);
				bDllInjectionSuccess = TRUE;
			CloseHandle(hProc);
		}
		if(!bDllInjectionSuccess)
			bGotFailure = TRUE;
		printf("%s\n",bDllInjectionSuccess?"OKAY":"FAIL");
	}
	
	//output summary for normal humans to read
	printf("\n");
	if((SetDbgRet==0)&&(bGotFailure==FALSE))
		printf("System is under control.\n");
	if((SetDbgRet!=0)||bGotFailure)
		printf("Applications are under control, but System services are not.\n");
	Sleep(DelayForUserReadingOutput_SECONDS*1000);
}