//////////////////////////////////////////////////////////////////////////////
//  File:       commonfun.cpp
//  Module:     INSIDER.DLL
//  Copyright 2005, WinBlox Project http://umbrella.name/upid/winblox
//
#define PROGRAMMING_MISTAKE_UNEXPECTED_NULL_POINTER 1101
char * GetFullFileNameInCurrentFolder(char * sFileId,char *sOutputBuff, size_t nSize)
//here is the plan for getting well-formed filename: 
//skip "\\?\" or "\\?\UNC\" at the beginning, GetFullPathName, strip out ending stream sign, strip out ending dot.
//we should have handled 8dot3 filename here by invoking GetLongPathNameA, but calling that function simply hangs system when trying to log out.
//so you should consider 8dot3 filename when coding the configuration file named "WBLIST.TXT"
//REF: http://msdn.microsoft.com/library/default.asp?url=/library/en-us/fileio/base/naming_a_file.asp
//REF: mspress/"writing secure code"/2nd edition/page#386/"attempt to canonicalize the name"
{
    if(sOutputBuff==NULL)
        ExitProcess(PROGRAMMING_MISTAKE_UNEXPECTED_NULL_POINTER);
    if(sFileId==NULL)
    {
        sOutputBuff[0] = 0;
        return sOutputBuff;
    }
    char * lp = NULL;
    DWORD dwRet = 0;
    _strlwr(sFileId);
    if(strstr(sFileId,"\\\\?\\")==sFileId)//if it begins with "\\?\"
    {
        sFileId = sFileId + strlen("\\\\?\\"); // skip "\\?\"
        if(strstr(sFileId,"unc\\")==sFileId)//if it begins with "unc\" after deleting the "\\?\"
        {
            sFileId = sFileId + 2; //skip "un"
            (*sFileId) = '\\'; //change "c" to slash
        }
    }
    dwRet = GetFullPathNameA(sFileId, nSize, sOutputBuff, &lp); 
    if(dwRet>FILENAME_STRMAXLEN)//if the return value is greater than nBufferLength, it means buffer is too small
    {
        sOutputBuff[0] = 0;
        return sOutputBuff;
    }
    lp = strstr(sOutputBuff,"::$"); //search for the stream sign
    if(lp!=NULL)
        (*lp) = 0;
    if(sOutputBuff[strlen(sOutputBuff)-1]=='.') //search for ending dot
        sOutputBuff[strlen(sOutputBuff)-1] = 0;
    return sOutputBuff;
}
char * GetCurrentUserName(void)
{
    if( g_sCurrentUserName[0]!=0 )
        return g_sCurrentUserName;
    DWORD dwSize = USERNAME_STRMAXLEN;
    if(  GetUserNameA(g_sCurrentUserName,&dwSize)==0 ) //If the function fails, the return value is zero.
        g_sCurrentUserName[0] = 0;
    return g_sCurrentUserName;
}
char * GetMainModuleFileName(void)
{
    if(g_sMainModuleFileName[0]!=0 ) // it's already initialized.
        return g_sMainModuleFileName;
    DWORD dwRet = GetModuleFileNameExA(
        GetCurrentProcess(), //GetCurrentProcess function retrieves a pseudo handle for the current process. no need to close this handle
        NULL, //hModule :[in] Handle to the module. If this parameter is NULL, GetModuleFileNameEx returns the path of the executable file of the process specified in hProcess.
        g_sMainModuleFileName,
        FILENAME_STRMAXLEN
        );
    if(dwRet == 0)//If the function fails, the return value is zero
    {
        g_sMainModuleFileName[0] = 0;
        return g_sMainModuleFileName;
    }
    return g_sMainModuleFileName;
}
char * StripOutEndingNewLineChar(char * buff)
{
    if(buff==NULL)
        ExitProcess(PROGRAMMING_MISTAKE_UNEXPECTED_NULL_POINTER);
    unsigned int nBuffLen = strlen(buff);
    if(nBuffLen == 0)
        return buff;
    if(buff[nBuffLen-1] == 0x0a)//if ending with 0x0a
    {
        buff[nBuffLen-1] = 0;//delete the ending 0x0a char
        nBuffLen -- ;
    }
    if(buff[nBuffLen-1] == 0x0d)//now, ending 0x0a was stripped out, but if it still ends with 0x0d...
        buff[nBuffLen-1] = 0;
    return buff;
}
char * GetFullFileNameInInsiderFolder(char * sFileId,char *sOutputBuff, size_t  nSize)
{
    if(sOutputBuff==NULL)
        ExitProcess(PROGRAMMING_MISTAKE_UNEXPECTED_NULL_POINTER);
    if(sFileId==NULL)
    {
        sOutputBuff[0] = 0;
        return sOutputBuff;
    }
    if(strstr(sFileId,":\\")!=NULL) //it's already full filename
        if(  nSize>(strlen(sFileId)+1)  ) //enough space?
        {
            strncpy(sOutputBuff,sFileId,nSize);
            return sOutputBuff;
        }
        else
        {
            sOutputBuff[0] = 0;
            return sOutputBuff;
        }
    if(g_sInsiderFolder[0]==0)
    {
        DWORD dwRet = GetModuleFileNameExA(
                        GetCurrentProcess(), //GetCurrentProcess function retrieves a pseudo handle for the current process. no need to close this handle
                        g_hInstance,
                        g_sInsiderFolder,
                        FILENAME_STRMAXLEN
                        );
        if(dwRet == 0)//If GetModuleFileNameEx failed
        {
            g_sInsiderFolder[0] = 0;
            sOutputBuff[0] = 0;
            return sOutputBuff;
        }
        //GetModuleFileNameEx succeeded :-) now get the folder out of filename
        if(strstr(g_sInsiderFolder,"\\")==NULL)//GetModuleFileNameEx didn't return a full path. we can't trust billg's api too much.
        {
            g_sInsiderFolder[0] = 0;
            sOutputBuff[0] = 0;
            return sOutputBuff;
        }
        DWORD i = 0;
        for(i=strlen(g_sInsiderFolder)-1;i>0;i--)//now let's change the last slash to NULL
            if(g_sInsiderFolder[i]=='\\')
            {
                g_sInsiderFolder[i] = 0;
                break;
            }
    }
    int nRet = _snprintf(sOutputBuff,nSize,"%s\\%s",g_sInsiderFolder,sFileId);
    if(nRet < 0 )  //number of bytes required to store the data exceeds "nSize"
    {
        sOutputBuff[0] = 0;
        return sOutputBuff;
    }
    return sOutputBuff;
}
